# Sacha van Albada, Nov 2014

# Plot results of parameter scan for scaling the microcircuit model
# of Potjans & Diesmann (Cereb Cortex, 2014)
# Scaling synaptic weights as 1/sqrt(K)

# Data created using param_scan.py

import os
import numpy
import pylab
import matplotlib.pyplot as p
import sys
import microcircuit_analysis as ana
import csv
import pickle
from matplotlib import ticker

scale_type = 'w_and_ext'

output_base_dir = '../data/spiking/microcircuit_param_scan/'

sim_t = 30000.

rec_type = 'abs' # or 'frac'
# depending on rec_type, rec_spikes and rec_vm are interpreted as fractions or absolute numbers of neurons recorded
rec_spikes = 50
rec_vm = 0

figurepath = '../figures/'

# Time interval for rate, CV_ISI, and synchrony calculation
tmin = 200.
tmax = sim_t
# Time step for synchrony calculation
dt = 1 # ms

# parameters of covariance functions
tau_max = 100.
delta_tau = 0.5
cov_t = numpy.arange(-tau_max,tau_max+delta_tau/2,delta_tau)

n_pops = 8

pickle_data = False # whether to read data from original files and pickle them

n_neuron_min = 0.05
indegree_min = 0.05
n_neuron_max = 1.05
indegree_max = 1.05
n_neuron_step = 0.05
indegree_step = 0.05
n_neuron_scaling_factor_vals = numpy.arange(n_neuron_min, n_neuron_max, n_neuron_step)
indegree_scaling_factor_vals = numpy.arange(indegree_min, indegree_max, indegree_step)

pops = ['L23E', 'L23I', 'L4E', 'L4I', 'L5E', 'L5I', 'L6E', 'L6I']
pop_labels = {'L23E':'2/3E', 'L23I':'2/3I', 'L4E':'4E', 'L4I':'4I', 'L5E':'5E', 'L5I':'5I', 'L6E':'6E', 'L6I':'6I'}

if pickle_data:
    # Check for missing data
    for N in n_neuron_scaling_factor_vals:
        for K in indegree_scaling_factor_vals:
            os.chdir(output_base_dir + scale_type + '/' + str(N) + '_' + str(K))
            files = os.listdir('.')
            if 'qsub' not in files:
                print 'Simulation not finished for ' + str(N) + '_' + str(K)
    
    all_pop_rates = {}
    all_pop_cv_isi = {}
    all_pop_sync = {}
    all_pop_cov = {}  # covariance functions
    for pop in pops:
        all_pop_rates[pop] = numpy.zeros([len(n_neuron_scaling_factor_vals), len(indegree_scaling_factor_vals)]) + numpy.nan
        all_pop_cv_isi[pop] = numpy.zeros([len(n_neuron_scaling_factor_vals), len(indegree_scaling_factor_vals)]) + numpy.nan
        all_pop_sync[pop] = numpy.zeros([len(n_neuron_scaling_factor_vals), len(indegree_scaling_factor_vals)]) + numpy.nan
        all_pop_cov[pop] = {}
        for pop2 in pops:
            all_pop_cov[pop][pop2] = numpy.zeros([len(n_neuron_scaling_factor_vals), len(indegree_scaling_factor_vals), len(cov_t)]) + numpy.nan

    # load data and compute rates, CV ISIs, and synchrony
    for i, N in enumerate(n_neuron_scaling_factor_vals):
        for j, K in enumerate(indegree_scaling_factor_vals):
            print 'N: ', N, ', K: ', K
            os.chdir(output_base_dir + scale_type + '/' + str(N) + '_' + str(K))
            a = ana.data(simtime=sim_t, data_type=['spikes'], rec_type = rec_type, spike_rec = rec_spikes, vm_rec = rec_vm)
            if a.spike_data:
                a.create_pop_rates(tmin, tmax)
                a.create_pop_cv_isi(tmin, tmax)
                a.create_pop_sync(tmin, tmax, dt)
                for pop in a.default_pops_flat:
                    all_pop_rates[pop][i,j] = a.pop_rates[pop]
                    all_pop_cv_isi[pop][i,j] = a.pop_cv_isi[pop]
                    all_pop_sync[pop][i,j] = a.pop_sync[pop]

    # load covariances
    for i, N in enumerate(n_neuron_scaling_factor_vals):
        for j, K in enumerate(indegree_scaling_factor_vals):
            print 'N: ', N, ', K: ', K
            os.chdir(output_base_dir + scale_type + '/' + str(N) + '_' + str(K))
            if 'covariances.dat' in os.listdir('.'):
                with open('covariances.dat', 'rb') as csvfile:
                    l = csv.reader(csvfile, delimiter='\n')
                    for row in l:
                        if row:
                            if row[0][0]=='L':
                                pop1 = row[0].split('-')[0].replace(' ','')
                                pop2 = row[0].split('-')[1].replace(' ','')
                                index = 0
                            if row[0][0] not in ('t', 'd', 'L'):
                                all_pop_cov[pop1][pop2][i,j][index] = float(row[0])
                                index += 1
                
    
    # save data to file
    os.chdir(output_base_dir + scale_type)
    
    f = open('rates_'+ scale_type +'.pkl', 'wb')
    pickle.dump(all_pop_rates, f)
    f.close()
    
    f = open('cv_isi_'+ scale_type +'.pkl', 'wb')
    pickle.dump(all_pop_cv_isi, f)
    f.close()
    
    f = open('sync_'+ scale_type +'.pkl', 'wb')
    pickle.dump(all_pop_sync, f)
    f.close()
    
    f = open('cov_'+ scale_type +'.pkl', 'wb')
    pickle.dump(all_pop_cov, f)
    f.close()    

#------------------------------------------------------
if not pickle_data:
    # reading back the data
    os.chdir(output_base_dir)
    
    f = open('rates_'+ scale_type +'.pkl', 'r')
    all_pop_rates = pickle.load(f)
    f.close()
    
    f = open('cv_isi_'+ scale_type +'.pkl', 'r')
    all_pop_cv_isi = pickle.load(f)
    f.close()
    
    f = open('sync_'+ scale_type +'.pkl', 'r')
    all_pop_sync = pickle.load(f)
    f.close()
    

# remove inf values from synchrony
for pop in pops:
    all_pop_sync[pop][numpy.isinf(all_pop_sync[pop])]=numpy.nan

#------------------------------------------------------
# plot rates, CV_ISIs, and synchrony

cmap = pylab.cm.Greys # get a list of possible colormaps with p.cm.datad.keys(). Use cmap = pylab.cm.jet for color version
cmap.set_bad('w',1.)

interp = 'bicubic'
aspect = 'auto'

fig = p.figure(figsize=(8,12))

for i, pop in enumerate(pops):
    print i, pop
    ax = fig.add_subplot(n_pops,1,i+1, position=[0.12,(n_pops-i)/(1.2*n_pops), 0.25, 1./(1.35*n_pops)])
    vmin = numpy.nanmin(all_pop_rates[pop])
    vmax = numpy.nanmax(all_pop_rates[pop])
    masked_array = numpy.ma.array(all_pop_rates[pop], mask=numpy.isnan(all_pop_rates[pop]))
    im = ax.imshow(masked_array.T, cmap=cmap, interpolation=interp, origin='lower', aspect=aspect)
    cb = p.colorbar(im)
    tick_locator = ticker.MaxNLocator(nbins=5)
    cb.locator = tick_locator
    cb.update_ticks()
    p.xticks((1,7,13,19), ('0.1','0.4','0.7','1'))
    if i==0:
        p.title('Firing rate (spikes/s)')
    if i==4:
        p.text(-7, 30, 'K scale factor', rotation='vertical')
    if i!=n_pops-1:
        p.xticks([])
    p.text(-11, 10, pop_labels[pop])
    (n_neuron_max-n_neuron_min)/2
    # standard ax.get_xlim() is (-0.5, 19.5) corresponding to n_neuron_scaling_factor_vals
    # and similarly for indegree_scaling_factor_vals
    p.yticks((1,7,13,19), ('0.1','0.4','0.7','1'))

for i, pop in enumerate(pops):
    print i, pop
    ax = fig.add_subplot(n_pops,1,i+1, position=[0.4,(n_pops-i)/(1.2*n_pops), 0.25, 1./(1.35*n_pops)])
    vmin = numpy.nanmin(all_pop_cv_isi[pop])
    vmax = numpy.nanmax(all_pop_cv_isi[pop])
    masked_array = numpy.ma.array(all_pop_cv_isi[pop], mask=numpy.isnan(all_pop_cv_isi[pop]))
    im = ax.imshow(masked_array.T, cmap=cmap, interpolation=interp, origin='lower', aspect=aspect)
    cb = p.colorbar(im)
    tick_locator = ticker.MaxNLocator(nbins=5)
    cb.locator = tick_locator
    cb.update_ticks()
    p.xticks((1,7,13,19), ('0.1','0.4','0.7','1'))
    p.yticks([])
    if i==0:
        p.title('CV ISI')
    if i==n_pops-1:
        p.xlabel('N scale factor')
    if i!=n_pops-1:
        p.xticks([])

for i, pop in enumerate(pops):
    print i, pop
    ax = fig.add_subplot(n_pops,1,i+1, position=[0.69,(n_pops-i)/(1.2*n_pops), 0.25, 1./(1.35*n_pops)])
    vmin = numpy.nanmin(all_pop_sync[pop])
    vmax = numpy.nanmax(all_pop_sync[pop])
    masked_array = numpy.ma.array(all_pop_sync[pop], mask=numpy.isnan(all_pop_sync[pop]))
    im = ax.imshow(masked_array.T, cmap=cmap, interpolation=interp, origin='lower', aspect=aspect)
    cb = p.colorbar(im)
    tick_locator = ticker.MaxNLocator(nbins=5)
    cb.locator = tick_locator
    cb.update_ticks()
    p.xticks((1,7,13,19), ('0.1','0.4','0.7','1'))
    p.yticks([])
    if i==0:
        p.title('Synchrony')
    if i!=n_pops-1:
        p.xticks([])

os.chdir('../..')
p.savefig(figurepath + 'fig3.eps')

