# Sacha van Albada, Nov 2014

# Covariance functions for different ways of downscaling 
# the microcircuit model of Potjans & Diesmann (Cereb Cortex, 2014) 

from plotfuncs import *
import pickle
import numpy

base_dir = '../data/spiking/microcircuit_param_scan/'

figurepath = '../figures/'

# parameters of covariance functions
tau_max = 100.
delta_tau = 0.5
cov_t = numpy.arange(-tau_max,tau_max+delta_tau/2,delta_tau)
zero_index = numpy.where(cov_t==0)[0][0]

K_scaling = 0.9
# another K_scaling to additionally plot for w_and_ext, to show that it also eventually changes correlation shapes
K_scaling2 = 0.3

K_scaling_w_and_ext = numpy.arange(0.05, 1.05, 0.05)
K_scaling_g_and_ext = numpy.arange(0.05, 1.05, 0.05)
K_scaling_w_and_g = numpy.arange(0.4, 1.05, 0.05)
K_scaling_w_mean_var_ext = numpy.arange(0.9, 1.95, 0.05)

w_and_ext_index = numpy.where(abs(K_scaling_w_and_ext-K_scaling)<0.0001)[0][0]
w_and_ext_index2 = numpy.where(abs(K_scaling_w_and_ext-K_scaling2)<0.0001)[0][0]
g_and_ext_index = numpy.where(abs(K_scaling_g_and_ext-K_scaling)<0.0001)[0][0]
w_and_g_index = numpy.where(abs(K_scaling_w_and_g-K_scaling)<0.0001)[0][0]
w_mean_var_ext_index = numpy.where(abs(K_scaling_w_mean_var_ext-K_scaling)<0.0001)[0][0]

# load raw but centered count covariances (nbin*counts^2/bin^2)
# calculated as cov_center[target_index][source_index] = numpy.array(list(cov[target_index][source_index][::-1]) + list(cov[source_index][target_index][1:])) - n_events_target*n_events_source*delta_tau/(t-0.1-tau_max)
# with delta_tau=0.5 ms, tau_max=100 ms, t=30 s
# and n_events_target and n_events_source contain total nrs of spikes, starting from Tstart=tau_max=100 ms
# Use spike rates recorded from 50 neurons in each population for normalization.

f = open(base_dir + '/cov_w_and_ext.pkl', 'r')
cov_w_and_ext = pickle.load(f)
f.close()

f = open(base_dir + '/rates_w_and_ext.pkl', 'r')
rates_w_and_ext = pickle.load(f)
f.close()

f = open(base_dir + '/cov_g_and_ext.pkl', 'r')
cov_g_and_ext = pickle.load(f)
f.close()

f = open(base_dir + '/rates_g_and_ext.pkl', 'r')
rates_g_and_ext = pickle.load(f)
f.close()

f = open(base_dir + '/cov_w_and_g.pkl', 'r')
cov_w_and_g = pickle.load(f)
f.close()

f = open(base_dir + '/rates_w_and_g.pkl', 'r')
rates_w_and_g = pickle.load(f)
f.close()

f = open(base_dir + '/cov_w_mean_var_ext_noise.pkl', 'r')
cov_w_mean_var_ext = pickle.load(f)
f.close()

f = open(base_dir + '/rates_w_mean_var_ext_noise.pkl', 'r')
rates_w_mean_var_ext = pickle.load(f)
f.close()


pop1 = 'L23E'
pop2 = 'L23E'

# covariances were based on counts from all neurons
num_neurons = {'L23E':20683, 'L23I':5834,
  'L4E':21915, 'L4I':5479,
  'L5E':4850,  'L5I':1065,
  'L6E':14395, 'L6I':2948}

# time over which events were counted
T = 30000.-0.1-tau_max


# normalize to get c^2/r^2
# if you want to plot covariances for scaled N, adjust the normalization below
norm = (num_neurons[pop1]*num_neurons[pop2]*delta_tau/1000.*T/1000.)

cov_23e_23e_full = cov_w_and_ext[pop1][pop2][-1,-1,:]/norm/rates_w_and_ext[pop1][-1,-1]/rates_w_and_ext[pop2][-1,-1]
cov_23e_23e_w_and_ext = cov_w_and_ext[pop1][pop2][-1,w_and_ext_index,:]/norm/rates_w_and_ext[pop1][-1,w_and_ext_index]/rates_w_and_ext[pop2][-1,w_and_ext_index]
cov_23e_23e_w_and_ext2 = cov_w_and_ext[pop1][pop2][-1,w_and_ext_index2,:]/norm/rates_w_and_ext[pop1][-1,w_and_ext_index2]/rates_w_and_ext[pop2][-1,w_and_ext_index2]
cov_23e_23e_g_and_ext = cov_g_and_ext[pop1][pop2][-1,g_and_ext_index,:]/norm/rates_g_and_ext[pop1][-1,g_and_ext_index]/rates_g_and_ext[pop2][-1,g_and_ext_index]
cov_23e_23e_w_and_g = cov_w_and_g[pop1][pop2][-1,w_and_g_index,:]/norm/rates_w_and_g[pop1][-1,w_and_g_index]/rates_w_and_g[pop2][-1,w_and_g_index]
cov_23e_23e_w_mean_var_ext = cov_w_mean_var_ext[pop1][pop2][-1,w_mean_var_ext_index,:]/norm/rates_w_mean_var_ext[pop1][-1,w_mean_var_ext_index]/rates_w_mean_var_ext[pop2][-1,w_mean_var_ext_index]

scale = 1.0
width = 6.83 
n_horz_panels = 2
n_vert_panels = 2

panel_factory = create_fig_plos(1, scale, width, n_horz_panels, n_vert_panels)

xmin = -50
xmax = 50

axA = panel_factory.new_panel(0, 0, 'A')
plot(cov_t, cov_23e_23e_full, color=black, label='full scale')
plot(cov_t, cov_23e_23e_g_and_ext, color=midgray, label=r'$K=0.9 K_0$')
f0 = cov_23e_23e_full[zero_index]/cov_23e_23e_g_and_ext[zero_index]
plot(cov_t, cov_23e_23e_g_and_ext*f0, color=lightgray, label='fit to c(0)')

rcParams['legend.fontsize'] = scale*8

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{c}{r^2}$')
title(r'scaling $g$ and $m_x$')
legend(loc=1)

axB = panel_factory.new_panel(1, 0, 'B')
plot(cov_t, cov_23e_23e_full, color=black, label='full scale')
plot(cov_t, cov_23e_23e_w_and_g, color=midgray, label=r'$K=0.9 K_0$')
f0 = cov_23e_23e_full[zero_index]/cov_23e_23e_w_and_g[zero_index]
plot(cov_t, cov_23e_23e_w_and_g*f0, color=lightgray, label='fit to c(0)')

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{c}{r^2}$')
title(r'scaling $J$ and $g$')

axC = panel_factory.new_panel(0, 1, 'C')
plot(cov_t, cov_23e_23e_full, color=black)
plot(cov_t, cov_23e_23e_w_and_ext, color=midgray)
plot(cov_t, cov_23e_23e_w_and_ext2, linewidth=0.5, color=midgray, label=r'$K=0.3 K_0$')
f0 = cov_23e_23e_full[zero_index]/cov_23e_23e_w_and_ext2[zero_index]
plot(cov_t, cov_23e_23e_w_and_ext2*f0, linewidth=0.5, color=lightgray, label='fit to c(0)')
f0 = cov_23e_23e_full[zero_index]/cov_23e_23e_w_and_ext[zero_index]
plot(cov_t, cov_23e_23e_w_and_ext*f0, color=lightgray)

legend(loc=1)

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{c}{r^2}$')
title(r'scaling $J$ and $m_x, J \propto 1/\sqrt{K}$')

axD = panel_factory.new_panel(1, 1, 'D')
plot(cov_t, cov_23e_23e_full, color=black, label='full scale')
plot(cov_t, cov_23e_23e_w_mean_var_ext, color=midgray, label=r'$K=0.9 K_0$')
f0 = cov_23e_23e_full[zero_index]/cov_23e_23e_w_mean_var_ext[zero_index]
plot(cov_t, cov_23e_23e_w_mean_var_ext*f0, color=lightgray, label='fit to c(0)')

xlim([xmin,xmax])
xlabel(r'$\Delta$' + ' (ms)')
ylabel(r'$\frac{c}{r^2}$')
title(r'scaling $J$ and $\sigma_x, J \propto 1/K$')

savefig(figurepath + 'fig4.eps')
