# Moritz Helias, 2014

# Randomly connected recurrent binary network with an excitatory (E), inhibitory (I), and external (X) population

import nest
import pylab
import numpy

from parameters_ginzburg import *


def simulate(N, N0, n_virtual_procs):

    N_rec = N / 2

    suffix = 'dist_Kin_scale' + str(N)

    # rescale synaptic weights
    j_EE1 = j_EE * numpy.sqrt(1.*N0/N)
    j_II1 = j_II * numpy.sqrt(1.*N0/N)
    j_EI1 = j_EI * numpy.sqrt(1.*N0/N)
    j_IE1 = j_IE * numpy.sqrt(1.*N0/N)
    j_EX1 = j_EX * numpy.sqrt(1.*N0/N)
    j_IX1 = j_IX * numpy.sqrt(1.*N0/N)

    # threshold stays the same, because completely balanced

    nest.ResetKernel()
    nest.SetKernelStatus({"resolution": h, "print_time": False, "overwrite_files": True, 'total_num_virtual_procs': n_virtual_procs})

    # this line is crucial to prevent multapses from being created
    # otherwise the decoding of up-/downtransitions will go wrong
    nest.sr("/RandomConvergentConnect << /allow_multapses false >> SetOptions")


    nest.SetDefaults("multimeter",
                    {"withtime": False,
                     "withgid": False,
                     "to_memory": False,
                     "to_file": True,
                     "record_from": ["h"],
                     "interval": dt_samp})

    # create recorders for activity
    multi_exc1 = nest.Create("multimeter")
    multi_exc2 = nest.Create("multimeter")
    multi_inh1 = nest.Create("multimeter")
    multi_inh2 = nest.Create("multimeter")
    multi_ext = nest.Create("multimeter")


    print "Building network"

    # mean activity m_x
    nest.SetDefaults('ginzburg_neuron', {'theta': 0.0, 'tau_m': tau_m, 'c_1': 0.0, 'c_2': 2.*m_x, 'c_3': 1.0})

    # external neurons
    nrnX = nest.Create('ginzburg_neuron', N)

    nest.SetDefaults('ginzburg_neuron', {'theta': theta, 'tau_m': tau_m, 'c_1': 0.0, 'c_2': 1.0, 'c_3': beta})


    # excitatory neurons
    nrnE = nest.Create('ginzburg_neuron', N)

    # inhibitory neurons
    nrnI = nest.Create('ginzburg_neuron', N)


    # these neurons are only used to record the activity of subpopulations in the network
    nrn_pre_sum_exc1 = nest.Create('ginzburg_neuron')
    nrn_pre_sum_exc2 = nest.Create('ginzburg_neuron')

    nrn_pre_sum_inh1 = nest.Create('ginzburg_neuron')
    nrn_pre_sum_inh2 = nest.Create('ginzburg_neuron')

    nrn_pre_sum_ext = nest.Create('ginzburg_neuron')



    nest.SetStatus(multi_exc1, {"label": datapath + "ginzburg-exc1" + suffix})

    nest.SetStatus(multi_exc2, {"label": datapath + "ginzburg-exc2" + suffix})

    nest.SetStatus(multi_inh1, {"label": datapath + "ginzburg-inh1" + suffix})

    nest.SetStatus(multi_inh2, {"label": datapath + "ginzburg-inh2" + suffix})

    nest.SetStatus(multi_ext, {"label": datapath + "ginzburg-ext" + suffix})


    print "Creating recurrent connectivity"

    numpy.random.seed(1)

    # recurrent connectivity from excitatory to excitatory neurons
    nest.SetDefaults('static_synapse', {'weight': j_EE1, 'delay': d, 'receptor_type': 0})


    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnE):
        nest.RandomConvergentConnect(nrnE, [n], K[i])

    # recurrent connectivity from inhibitory to excitatory neurons
    nest.SetDefaults('static_synapse', {'weight': j_EI1, 'delay': d, 'receptor_type': 0})

    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnE):
        nest.RandomConvergentConnect(nrnI, [n], K[i])

    # recurrent connectivity from excitatory to inhibitory neurons
    nest.SetDefaults('static_synapse', {'weight': j_IE1, 'delay': d, 'receptor_type': 0})

    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnI):
        nest.RandomConvergentConnect(nrnE, [n], K[i])

    # recurrent connectivity from inhibitry to inhibitory neurons
    nest.SetDefaults('static_synapse', {'weight': j_II1, 'delay': d, 'receptor_type': 0})

    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnI):
        nest.RandomConvergentConnect(nrnI, [n], K[i])

    # recurrent connectivity from external to excitatory neurons
    nest.SetDefaults('static_synapse', {'weight': j_EX1, 'delay': d, 'receptor_type': 0})

    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnE):
        nest.RandomConvergentConnect(nrnX, [n], K[i])

    # recurrent connectivity from external to inhibitory neurons
    nest.SetDefaults('static_synapse', {'weight': j_IX1, 'delay': d, 'receptor_type': 0})

    K = numpy.random.binomial(N, p, N)
    for i,n in enumerate(nrnI):
        nest.RandomConvergentConnect(nrnX, [n], K[i])

    print "Connecting devices."

    nest.SetDefaults('static_synapse', {'weight': 1.0, 'delay': h})

    nest.ConvergentConnect(nrnE[:N_rec], nrn_pre_sum_exc1)
    nest.ConvergentConnect(nrnE[N/2:N/2+N_rec], nrn_pre_sum_exc2)

    nest.ConvergentConnect(nrnI[:N_rec], nrn_pre_sum_inh1)
    nest.ConvergentConnect(nrnI[N/2:N/2+N_rec], nrn_pre_sum_inh2)

    nest.ConvergentConnect(nrnX, nrn_pre_sum_ext)


    nest.Connect(multi_exc1, nrn_pre_sum_exc1)
    nest.Connect(multi_exc2, nrn_pre_sum_exc2)
    nest.Connect(multi_inh1, nrn_pre_sum_inh1)
    nest.Connect(multi_inh2, nrn_pre_sum_inh2)
    nest.Connect(multi_ext, nrn_pre_sum_ext)

    nest.Simulate(T)



