# Sacha van Albada, Nov 2014

# Script for running g_gamma_K_scaling.py on a cluster
# Adjust to your system as necessary

import os
import glob
import numpy

# Scale spiking network while preserving the integrated covariances in Eq. 7 of Helias et al. (2013) New J Phys, as well as L, which preserves the shapes of the covariance functions.

procs = [24, 24, 24, 24, 24, 24, 48]
nodes = [1, 1, 1, 1, 1, 1, 2]
Nrecs =[1000, 1000, 1000, 1000, 1000, 1000, 1000] # there need to be at least 2*Nrecs[i] neurons in each population
NEs = [2500, 3500, 5000, 7100, 10000, 14100, 20000]
mem = [1250, 1750, 2500, 3550, 5000, 7050, 10000]

g = [0.97889182058047486, 1.5149700598802394, 2.3185840707964607, 3.4467640918580376, 5., 7.1976744186046506, 10.363636363636363]
gamma = [1.358490566037736, 0.85770750988142286, 0.55057251908396931, 0.36583888552392485, 0.25, 0.17259019924609587,  0.11933479532163745]
epsilon = [0.30320000000000003, 0.23857142857142857, 0.18079999999999999, 0.13492957746478873, 0.1, 0.073191489361702125, 0.0528]


J = 0.1
mu = 15.
sigma = 10.
nu0 = 23.6
delay=0.1

simname = 'g_gamma_K_scaling'

workingdir = '/users/albada/Documents/downscale_manuscript/py'
qsubfile = '/users/albada/Documents/downscale_manuscript/py/qsub'

os.system('cd ' + workingdir)

# create one pyf and one jdf file per job

for k,NE in enumerate(NEs):

       # write a python file to execute
       pyf_name = './pyf_'+str(k)+'.py'
       f = open( pyf_name , 'w')
       f.write( 'import g_gamma_K_scaling\n' )
       f.write( 'g_gamma_K_scaling.simulate('
                       +str(g[k])+','\
                       +str(gamma[k])+','\
                       +str(epsilon[k])+','\
                       +str(J)+','\
                       +str(NE)+','\
                       +str(delay)+','\
                       +str(mu)+','\
                       +str(sigma)+','\
                       +str(nu0)+','\
                       +str(Nrecs[k])+','\
                       +str(procs[k]) +') \n' )

       f.close()
       
       # write a job description file for qsub
       jdf_name = './job_'+str(k)+'.jdf' 
       f = open( jdf_name , 'w')
       f.write( '#!/bin/bash \n')
       f.write( '#PBS -N '+simname+'_'+str(k)+'\n' )  #set job name
       f.write( '#PBS -d '+workingdir+'\n' )    #working directory
       f.write( '#PBS -o '+qsubfile+'\n' )      #output directory to send standard/error output stream to
       f.write( '#PBS -j oe \n' )               #join error stream and output stream to obtain a single file in the end
       f.write( '#PBS -l nodes=' + str(nodes[k]) + ':ppn=24\n' )    #request a total of nodes*8 processors for this job
       f.write( '#PBS -l walltime=4:00:00,mem=' + str(mem[k]) + 'mb \n' ) #request processor time and memory
       f.write('export PYTHONPATH=/users/albada/10kcollaps_install.hambach/lib64/python2.6/site-packages\n')
       f.write('. /usr/local/mpi/openmpi/1.4.3/gcc64/bin/mpivars_openmpi-1.4.3_gcc64.sh\n') 
       f.write('mpirun -np ' + str(procs[k]) + ' python ' + pyf_name + '\n')
       f.close()
       
       # submit job
       os.system('qsub ' + jdf_name)


