# Moritz Helias, 2014

# Code for running ginzburg_network_dist_Kin_scale.py on a cluster
# Adjust to your system as necessary

import os
import glob
import numpy




##################################################
# queue specific

nodes = 20          # how many nodes to use
mpiprocs = 80       # how many processes to use
core_per_node = 24  # how many processors per node
threads_per_process = 6

workingdir = '/scratch/albada/intricorr/py'
qsubfile = '/scratch/albada/intricorr/qsub'

wallclock_limit = "20:00:00"

os.system('cd ' + workingdir)

N0 = 8192

nvp = nodes*core_per_node
mem_tot = 2.2*nvp

for k,N in enumerate([10000, 30000, 100000]):

       simname = 'dist_scale' + str(N)

       # write a python file to execute

       pyf_name = './pyf_'+str(k)+'.py'
       f = open( pyf_name , 'w')
       f.write('import  ginzburg_network_dist_Kin_scale as gn\n')
       f.write('gn.simulate(' + str(N) + ',' + str(N0) + ',' + str(nvp) + ')' )
       f.close()

       # write a job description file for qsub

       jdf_name = './job_'+str(k)+'.jdf' 
       f = open( jdf_name , 'w')
       f.write( '#!/bin/bash \n')
       f.write( '#PBS -N '+simname+'\n' )  #set job name
       f.write( '#PBS -d '+workingdir+'\n' )    #working directory
       f.write( '#PBS -o '+qsubfile+'\n' )      #output directory to send standard/error output stream to
       f.write( '#PBS -j oe \n' )               #join error stream and output stream to obtain a single file in the end
       f.write( '#PBS -l nodes=' + str(nodes) + ':ppn='+str(core_per_node)+'\n' )    #request a total of nodes*proc_per_node processors for this job
       f.write( '#PBS -l mem=%dGB\n' % mem_tot )    #request a total of nodes*proc_per_node processors for this job
       f.write( '#PBS -l walltime=' + wallclock_limit + '\n' ) #request for processor time
       f.write('export PYTHONPATH=/users/albada/10kproject_install.hambach/lib64/python2.6/site-packages\n')
       f.write('. /usr/local/mpi/openmpi/1.4.3/gcc64/bin/mpivars_openmpi-1.4.3_gcc64.sh\n') 
       f.write('mpirun -np ' + str(mpiprocs) + ' -bynode -bind-to-core -cpus-per-proc ' + str(threads_per_process) + ' -report-bindings ' + 'python ' + pyf_name + '\n')


       f.close()
       
       # submit job
       os.system('qsub ' + jdf_name)
