# Sacha van Albada, Nov 2014

from params import *
import random
from scipy import stats

def MultinomialConnect(sim, pop1, pop2, K, weight_mean, weight_sd, delay_mean, delay_sd):
    """NEST-specific function connecting two populations with multapses and a fixed total number of synapses"""
    if not K:
        return
    source_neurons = list(pop1.all_cells)
    target_neurons = list(pop2.all_cells)
    n_syn = int(round(K*len(target_neurons)))
    # weights are multiplied by 1000 because NEST uses pA whereas PyNN uses nA
    # RandomPopulationConnectD is called on each process with the full sets of source and target neurons, and internally only connects the target neurons on the current process

    wd_dict = {'weight_m': 1000*weight_mean,
               'weight_s': 1000*weight_sd,
               'delay_m': delay_mean,
               'delay_s': delay_sd}
              }

    sim.nest.sli_push(source_neurons)
    sim.nest.sli_push(target_neurons)
    sim.nest.sli_push(n_syn)
    sim.nest.sli_push(wd_dict)
    sim.nest.sli_run("/static_synapse RandomPopulationConnectD")

    if save_connections:
        # get connections to target on this MPI process
        conn = sim.nest.GetConnections(source = source_neurons, target=target_neurons)
        conns = sim.nest.GetStatus(conn, ['source', 'target', 'weight', 'delay'])
        f = open(output_dir +  '/' + pop1.label + "_" + pop2.label + '.conn' + str(sim.rank()), 'w')
        for c in conns:
            print >> f, str(c).replace('(','').replace(')','').replace(', ', '\t')
        f.close()
