# Sacha van Albada, Nov 2014

# Loop over parameters, writing a job description file and custom_params.py, and then submitting the job for each set of parameters

import os
import numpy

nodes = 1          # how many compute nodes to use
procs = 12         # how many processor cores to use in total
proc_per_node = 12 # how many processors to use per compute node
mem = 24000        # how much total memory required (in MB)

scale_type = 'w_and_ext'

simname = 'microcircuit_param_scan'

# where your python code for the microcircuit model resides
workingdir = '/users/albada/microcircuit/PyNN'
# output base directory
output_base_dir = '/scratch/albada/'
# output directory 
outputdir = output_base_dir + str(scale_type)

# create directory if it doesn't exist yet
if scale_type not in os.listdir(output_base_dir):
    os.system('mkdir ' + outputdir)

# python file to be executed by the queue
pyf_name = 'microcircuit.py'

# job description file
jdf_name = 'param_scan.jdf'


n_neuron_scaling_factor_vals = numpy.arange(0.05, 1.05, 0.05)
indegree_scaling_factor_vals = numpy.arange(0.05, 1.05, 0.05)

for i, N in enumerate(n_neuron_scaling_factor_vals):
    for j, K in enumerate(indegree_scaling_factor_vals):

        # output directory for this parameter combination
        this_outputdir = str(N) + '_' + str(K)
        full_outputdir = outputdir + '/' + this_outputdir
        # create directory if it doesn't exist yet
        if this_outputdir not in os.listdir(outputdir):
            os.system('mkdir ' + full_outputdir)

        # file into which error/stdout is written
        qsubfile = full_outputdir + '/qsub'
                
        # make directory 'PyNN' in output directory if it doesn't exist yet
        if 'PyNN' not in os.listdir(full_outputdir):
            os.system('mkdir ' + full_outputdir + '/PyNN')

        os.chdir(workingdir)

        # copy all the relevant files to the output directory
        os.system('cp microcircuit.py ' + full_outputdir + '/PyNN')
        os.system('cp connectivity.py ' + full_outputdir + '/PyNN')
        os.system('cp network.py ' + full_outputdir + '/PyNN')
        os.system('cp params.py ' + full_outputdir + '/PyNN')
        os.system('cp scaling.py ' + full_outputdir + '/PyNN')

        os.chdir(full_outputdir + '/PyNN')

        # write custom_params file, which will be invoked in the simulation after importing the default parameters
        f = open(full_outputdir + '/PyNN/custom_params.py', 'w')
        f.write('scale_type = "' + str(scale_type) + '"\n')
        f.write('n_neuron_scaling_factor = ' + str(N) + '\n')
        f.write('indegree_scaling_factor = ' + str(K) + '\n')
        f.write('output_dir = "' + full_outputdir + '"')
        f.close()
 
        this_pyf_name = full_outputdir  + '/PyNN/' + pyf_name

        # write job description file
        f = open(full_outputdir + '/PyNN/' + jdf_name , 'w')
        f.write( '#!/bin/bash \n')
        # set job name
        f.write( '#PBS -N '+simname+'\n' )
        # working directory
        f.write( '#PBS -d '+full_outputdir+'/PyNN\n' )
        # output file to send standard/error output stream to
        f.write( '#PBS -o '+qsubfile+'\n' )
        # join error stream and output stream to obtain a single file in the end
        f.write( '#PBS -j oe \n' )
        # request a total of nodes*processes_per_node for this job
        f.write('#PBS -l nodes=%i:ppn=%i\n' % (nodes, proc_per_node))
        # request processor time
        f.write( '#PBS -l walltime=120:00:00\n' )
        # request mem MB of memory
        f.write( '#PBS -l mem=' + str(mem) + 'mb \n' )
        f.write('export PYTHONPATH=/users/albada/10kproject_10711_install.hambach/lib64/python2.6/site-packages:/users/albada/pyNN_hambach/lib/python2.6/site-packages\n')
        f.write('. /usr/local/mpi/openmpi/1.4.3/gcc64/bin/mpivars_openmpi-1.4.3_gcc64.sh\n') 
        f.write('mpirun -np %i python %s\n' % (procs, this_pyf_name))
        f.close()

        # submit job
        os.system('qsub ' + jdf_name)

