# Sacha van Albada, Nov 2014

from params import *
import numpy

def get_indegrees(n_neurons):
    '''get in-degrees for each connection for the full-scale (1 mm^2) model'''
    K = numpy.zeros([n_layers*n_pops_per_layer, n_layers*n_pops_per_layer])
    for target_layer in layers:
        for target_pop in pops:
            for source_layer in layers:
                for source_pop in pops:
                    target_index = structure[target_layer][target_pop]
                    source_index = structure[source_layer][source_pop]
                    K[target_index][source_index] = round(numpy.log(1-conn_probs[target_index][source_index])/numpy.log((n_neurons[target_layer][target_pop]*n_neurons[source_layer][source_pop]-1.)/(n_neurons[target_layer][target_pop]*n_neurons[source_layer][source_pop])))/n_neurons[target_layer][target_pop]
    return K


def adjust_w_and_ext_to_K(K_full, K_scaling, w, DC):
    '''For scale_type w_and_ext'''
    I_ext = {}
    for target_layer in layers:
        I_ext[target_layer] = {}
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            x1 = 0
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    x1 += w[target_index][source_index]*K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
            x1_ext = w_ext*K_ext[target_layer][target_pop]*bg_rate
            w_new = w/numpy.sqrt(K_scaling)
            if scale_ext and 'poisson' in input_types:
                external_K_scaling = K_scaling
            else:
                external_K_scaling = 1.
            w_ext_new = w_ext/numpy.sqrt(external_K_scaling)
            # divide I_ext by 1000 to get it in nA
            I_ext[target_layer][target_pop] = 0.001*neuron_parameters['tau_syn_E']*((1.-numpy.sqrt(K_scaling))*x1 + (1.-numpy.sqrt(external_K_scaling))*x1_ext) + DC[target_layer][target_pop]
    return w_new, w_ext_new, I_ext


def adjust_g_and_ext_to_K(K_full, K_scaling, w, DC):
    '''For scale_type = g_and_ext. Effectively this changes the inhibitory weights, and makes these specific to the target population.'''
    g_new = {}
    I_ext = {}
    w_m_matrix = numpy.zeros([n_layers*n_pops_per_layer, n_layers*n_pops_per_layer])
    for target_layer in layers:
        g_new[target_layer] = {}
        I_ext[target_layer] = {}
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            x1 = {'E':0, 'I':0}
            x2 = {'E':0, 'I':0}
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    x1[source_pop] += w[target_index][source_index]*K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
                    x2[source_pop] += (w[target_index][source_index])**2*K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
            # if also adjusting the external Poisson input
            if scale_ext and 'poisson' in input_types:
                x1['E'] += w_ext*K_ext[target_layer][target_pop]*bg_rate
                x2['E'] += w_ext**2*K_ext[target_layer][target_pop]*bg_rate
            g_new[target_layer][target_pop] = -numpy.sqrt((g**2*(x2['I'] + x2['E']*(1.-K_scaling)))/(x2['I']*K_scaling))
            # divide I_ext by 1000 to get it in nA
            I_ext[target_layer][target_pop] = 0.001*neuron_parameters['tau_syn_E']*(x1['E']*(1.-K_scaling) + x1['I']*(1. - g_new[target_layer][target_pop]*K_scaling/g)) + DC[target_layer][target_pop]
    for target_layer in layers:
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    if source_pop == 'E':
                        w_m_matrix[target_index][source_index] = w[target_index][source_index]
                    else:
                        w_m_matrix[target_index][source_index] = g_new[target_layer][target_pop]*w_mean
    return w_m_matrix, I_ext



def adjust_w_and_g_to_K(K_full, K_scaling, w, equal_factor = True):
    ''' Calculate target-population-specific synaptic weights that approximately preserve the mean and variance of the population activities for the given full-scale in-degrees.'''
    g_new = {}
    w_new = {}
    iota = {}
    w_m_matrix = numpy.zeros([n_layers*n_pops_per_layer, n_layers*n_pops_per_layer])
    for target_layer in layers:
        g_new[target_layer] = {}
        w_new[target_layer] = {}
        iota[target_layer] = {}
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            x0 = {'E':0, 'I':0}
            x1 = {'E':0, 'I':0}
            x2 = {'E':0, 'I':0}
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    x0[source_pop] += K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
                    x1[source_pop] += w[target_index][source_index]*K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
                    x2[source_pop] += (w[target_index][source_index])**2*K_full[target_index][source_index]*full_mean_rates[source_layer][source_pop]
            # scaling all weights by the same factor
            if equal_factor:
                f = -x1['E']/x1['I']
                N_min = x2['E']*x2['I']*(f-1.)**2/((x2['E'] + x2['I']*f**2)*(x2['E']+x2['I']))
                a = K_scaling*(x2['E']+x2['I']*f**2)
                b = 2.*f*x2['I']*(1.-f)
                c = x2['I']/K_scaling*(f-1.)**2 - x2['E'] - x2['I']
                iota[target_layer][target_pop] = (-b + numpy.sqrt(b**2 - 4.*a*c))/(2.*a)
                g_new[target_layer][target_pop] = -((iota[target_layer][target_pop]*K_scaling-1.)*x1['E'] - x1['I'])*g/(iota[target_layer][target_pop]*K_scaling*x1['I'])
            # making all excitatory weights equal for the given target population
            else:
                N_min = (x1['E'] + x1['I'])**2/((x0['E']+x0['I'])*(x2['E']+x2['I']))
                w_new[target_layer][target_pop] = (numpy.sqrt(x0['E'])*(x1['E'] + x1['I']) + numpy.sqrt(x0['I']*K_scaling*(x0['E']+x0['I'])*(x2['E']+x2['I']) - x0['I']*(x1['E']+x1['I'])**2))/(K_scaling*numpy.sqrt(x0['E'])*(x0['E']+x0['I']))
                g_new[target_layer][target_pop] = (x1['E']+x1['I']-w_new[target_layer][target_pop]*K_scaling*x0['E'])/(w_new[target_layer][target_pop]*K_scaling*x0['I'])

            print 'Smallest possible in-degree scaling factor for ', target_layer, target_pop, ': ', N_min
    for target_layer in layers:
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    if source_pop == 'E':
                        if equal_factor:
                            w_m_matrix[target_index][source_index] = iota[target_layer][target_pop]*w[target_index][source_index]
                        else:
                            w_m_matrix[target_index][source_index] = w_new[target_layer][target_pop]
                    else:
                        if equal_factor:
                            w_m_matrix[target_index][source_index] = g_new[target_layer][target_pop]*iota[target_layer][target_pop]*w_mean
                        else:
                            w_m_matrix[target_index][source_index] = g_new[target_layer][target_pop]*w_new[target_layer][target_pop]

    return w_m_matrix



def adjust_w_mean_var_ext_poisson(K, w):
    '''Scaling that preserves firing rates and the shape of correlation functions, though not their scale unless n_neuron_scaling_factor=1. See Helias et al. (2013) New J Phys Eq. E.1. Version with Poisson input. Only upscaling is possible, since the inhibitory Poisson rate otherwise becomes negative.'''
    # Scale weights
    w_new = w/K_scaling
    # Determine new external Poisson input (both excitatory and inhibitory, in order to adjust the variance)
    K_ext_e = {}
    K_ext_i = {}
    for target_layer in layers:
        K_ext_e[target_layer] = {}
        K_ext_i[target_layer] = {}
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            x2_full = 0 # variance of local input to this population in full-scale network, ignoring tau
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    x2_full += (w[target_index][source_index])**2*K[target_index][source_index]*full_mean_rates[source_layer][source_pop]
            # Helias et al. (2013) Eq. (E.1) with correction 1+g^2 -> 1-g (the minus sign is because our g is negative, whereas it was positive in Helias et al.) and ignoring tau, which cancels in the numerator and denominator
            input_bal = x2_full*(1.-1./K_scaling)/(w_ext**2*(1.-g))
            print 'input_bal[', target_index, ']: ', input_bal
            print 'poisson_rate[', target_index, ']: ', K_ext[target_layer][target_pop]*bg_rate
            K_ext_e[target_layer][target_pop] = K_ext[target_layer][target_pop] + input_bal/bg_rate
            K_ext_i[target_layer][target_pop] = input_bal/(-g*bg_rate)
    return w_new, K_ext_e, K_ext_i



def adjust_w_mean_var_ext_noise(K, w):
    '''Scaling that preserves firing rates and the shape of correlation functions, though not their scale unless n_neuron_scaling_factor=1. See Helias et al. (2013) New J Phys Eq. E.1. Version with Gaussian noise input. Downscaling is possible until the external variance becomes zero. Since the external variance in the full-scale model is small compared to the intrinsic variance, this only enables a minimal amount of downscaling.'''
    # Scale weights
    w_new = w/K_scaling
    # Determine required mean and variance of external noise input (in nA; the mean is the same as before scaling)
    mu = {}
    sigma = {}
    for target_layer in layers:
        mu[target_layer] = {}
        sigma[target_layer] = {}
        for target_pop in pops:
            target_index = structure[target_layer][target_pop]
            mu[target_layer][target_pop] = K_ext[target_layer][target_pop]*bg_rate*w_ext*neuron_parameters['tau_syn_E']/1000.
            print 'mu[', target_layer, '][', target_pop, ']: ', mu[target_layer][target_pop]
            # variance of external input to this population in full-scale network
            x2_ext = w_ext**2*K_ext[target_layer][target_pop]*bg_rate*neuron_parameters['tau_syn_E']/1000.
            x2_local = 0 # variance of local input to this population in full-scale network
            for source_layer in layers:
                for source_pop in pops:
                    source_index = structure[source_layer][source_pop]
                    if source_pop == 'E':
                        x2_local += (w[target_index][source_index])**2*K[target_index][source_index]*full_mean_rates[source_layer][source_pop]*neuron_parameters['tau_syn_E']/1000.
                    else:
                        x2_local += (w[target_index][source_index])**2*K[target_index][source_index]*full_mean_rates[source_layer][source_pop]*neuron_parameters['tau_syn_I']/1000.
            print 'x2_ext[', target_layer, '][', target_pop, ']: ', x2_ext
            print 'x2_local[', target_layer, '][', target_pop, ']: ', x2_local
            print 'change in x2_local[', target_layer, '][', target_pop, ']: ', x2_local*(1.-1./K_scaling)
            sigma[target_layer][target_pop] = numpy.sqrt(x2_ext + x2_local*(1.-1./K_scaling))
            if numpy.isnan(sigma[target_layer][target_pop]):
                raise ValueError('negative variance')
            print 'sigma[', target_layer, '][', target_pop, ']: ', sigma[target_layer][target_pop]
    return w_new, mu, sigma


