# Parameters for binary_network_asymm.py

import numpy as np
import sys

n_virtual_procs = 4

msd = 87346 # master seed

theta = 0.    # mV threshold

tau_m = 10.0   # ms time constant of neuron

neuronmodel = 'mcculloch_pitts_neuron' # "ginzburg_neuron" or "mcculloch_pitts_neuron"

beta = 100.0   # slope for ginzburg neuron in mV^-1. Use a high slope to obtain saturation so that tanh is effectively a Heaviside function

input_type = 'noise' # 'binary' or 'noise'

T = 30000.0   # ms

# sizes of excitatory, inhibitory populations, and also of the external population unless input_type = binary
N_full = 5000
N_scaling = 0.3 # scaling factor for numbers of neurons
N = int(round(N_full*N_scaling))

scale_type = 'helias' # 'helias': J propto 1/K; 'vreeswijk': J propto 1/sqrt(K); 'keep_var_ext': J scaling such that extrinsic variance does not change
scale_K_equally = True # whether to scale the K for each pair of populations by the same factor

if scale_type in ['vreeswijk', 'keep_var_ext'] and not scale_K_equally:
    print 'scale type not implemented for unequal K scaling factors'
    sys.exit(1)

# changing J to approximately preserve (mu-theta)/sigma but not mu and sigma separately
J_scaling = 1.

p_EE_full = 0.1
p_EI_full = 0.2
p_IE_full = 0.3
p_II_full = 0.4

if scale_K_equally:
    K_scaling = 0.75  # scaling factor for in-degrees
    p_EE = p_EE_full*K_scaling/N_scaling
    p_EI = p_EI_full*K_scaling/N_scaling
    p_IE = p_IE_full*K_scaling/N_scaling
    p_II = p_II_full*K_scaling/N_scaling
else:
    p_EE = 0.1
    p_EI = 0.2
    p_IE = 0.3
    p_II = 0.4

K_EE_full = int(round(p_EE_full*N_full))
K_EI_full = int(round(p_EI_full*N_full))
K_IE_full = int(round(p_IE_full*N_full))
K_II_full = int(round(p_II_full*N_full))

K_EE = int(round(p_EE*N))
K_EI = int(round(p_EI*N))
K_IE = int(round(p_IE*N))
K_II = int(round(p_II*N))

j_EE_full = 3.
j_EI_full = -5.
j_IE_full = 3.
j_II_full = -6.

if input_type == 'noise':
    noise_mean_E_full = J_scaling*50. # to E neurons
    noise_mean_I_full = J_scaling*40. # to I neurons
    noise_std_E_full = J_scaling*60.
    noise_std_I_full = J_scaling*50.

    # mean activities in full-scale network
    m_E_full = 0.155393285933
    m_I_full = 0.0716317923179

    intrinsic_mean_E_full = K_EE_full*j_EE_full*m_E_full + K_EI_full*j_EI_full*m_I_full
    intrinsic_mean_I_full = K_IE_full*j_IE_full*m_E_full + K_II_full*j_II_full*m_I_full
    intrinsic_var_E_full = K_EE_full*j_EE_full**2*m_E_full*(1-m_E_full) + K_EI_full*j_EI_full**2*m_I_full*(1-m_I_full)
    intrinsic_var_I_full = K_IE_full*j_IE_full**2*m_E_full*(1-m_E_full) + K_II_full*j_II_full**2*m_I_full*(1-m_I_full)

if scale_type == 'helias':
    j_EE = J_scaling*j_EE_full*K_EE_full/float(K_EE)  # coupling strength E->E
    j_EI = J_scaling*j_EI_full*K_EI_full/float(K_EI) # coupling strength I->E
    j_IE = J_scaling*j_IE_full*K_IE_full/float(K_IE)  # coupling strength E->I
    j_II = J_scaling*j_II_full*K_II_full/float(K_II) # coupling strength I->I
if scale_type == 'vreeswijk':
    j_EE = J_scaling*j_EE_full*np.sqrt(K_EE_full/float(K_EE)) 
    j_EI = J_scaling*j_EI_full*np.sqrt(K_EI_full/float(K_EI))
    j_IE = J_scaling*j_IE_full*np.sqrt(K_IE_full/float(K_IE))
    j_II = J_scaling*j_II_full*np.sqrt(K_II_full/float(K_II))
if scale_type == 'keep_var_ext':
    j_EE = J_scaling*j_EE_full*np.sqrt(noise_std_E_full**2/(K_scaling**2*noise_std_E_full**2 + K_scaling*(K_scaling-1.)*intrinsic_var_E_full))
    j_EI = J_scaling*j_EI_full*np.sqrt(noise_std_E_full**2/(K_scaling**2*noise_std_E_full**2 + K_scaling*(K_scaling-1.)*intrinsic_var_E_full))
    j_IE = J_scaling*j_IE_full*np.sqrt(noise_std_I_full**2/(K_scaling**2*noise_std_I_full**2 + K_scaling*(K_scaling-1.)*intrinsic_var_I_full))
    j_II = J_scaling*j_II_full*np.sqrt(noise_std_I_full**2/(K_scaling**2*noise_std_I_full**2 + K_scaling*(K_scaling-1.)*intrinsic_var_I_full))

# external drive

if input_type == 'binary': # (note: no scaling implemented for this option):
    p_EX_full = 0.2
    p_IX_full = 0.3
    if scale_K_equally:
        p_EX = p_EX_full*K_scaling/N_scaling
        p_IX = p_IX_full*K_scaling/N_scaling
    else:
        p_EX = 0.2
        p_IX = 0.3
    K_EX_full = int(round(p_EX_full*N_full))
    K_IX_full = int(round(p_IX_full*N_full))
    K_EX = int(round(p_EX*N))
    K_IX = int(round(p_IX*N))
    if scale_type == 'helias':
        j_IX = 3.*K_IX_full/K_IX
        j_EX = 3.*K_EX_full/K_EX
    if scale_type == 'vreeswijk':
        j_IX = 3.*np.sqrt(K_IX_full/float(K_IX))
        j_EX = 3.*np.sqrt(K_EX_full/float(K_EX))
    m_x = 0.1     # mean activity of external neurons
    noise_mean_E = noise_mean_I = noise_std_E = noise_std_I = 0.

if input_type == 'noise':
    intrinsic_mean_E = K_EE*j_EE*m_E_full + K_EI*j_EI*m_I_full # assume that m_E and m_I are successfully preserved. If the scaling works as it should, this is self-consistent.
    intrinsic_mean_I = K_IE*j_IE*m_E_full + K_II*j_II*m_I_full
    intrinsic_var_E = K_EE*j_EE**2*m_E_full*(1-m_E_full) + K_EI*j_EI**2*m_I_full*(1-m_I_full)
    intrinsic_var_I = K_IE*j_IE**2*m_E_full*(1-m_E_full) + K_II*j_II**2*m_I_full*(1-m_I_full)

    if scale_type == 'helias':
        # preserve mu and sigma separately
        noise_mean_E = intrinsic_mean_E_full + noise_mean_E_full - intrinsic_mean_E
        noise_mean_I = intrinsic_mean_I_full + noise_mean_I_full - intrinsic_mean_I
        noise_std_E = np.sqrt(noise_std_E_full**2 + intrinsic_var_E_full - intrinsic_var_E)
        noise_std_I = np.sqrt(noise_std_I_full**2 + intrinsic_var_I_full - intrinsic_var_I)

    if scale_type == 'keep_var_ext':
        noise_std_E = noise_std_E_full
        noise_std_I = noise_std_I_full
        # adjust mu to keep (mu-theta)/sigma constant
        sigma_E_full = np.sqrt(intrinsic_var_E_full + noise_std_E_full**2)
        sigma_E = np.sqrt(intrinsic_var_E + noise_std_E**2)
        noise_mean_E = (noise_mean_E_full + intrinsic_mean_E_full - theta)*sigma_E/sigma_E_full + theta - intrinsic_mean_E
        sigma_I_full = np.sqrt(intrinsic_var_I_full + noise_std_I_full**2)
        sigma_I = np.sqrt(intrinsic_var_I + noise_std_I**2)
        noise_mean_I = (noise_mean_I_full + intrinsic_mean_I_full - theta)*sigma_I/sigma_I_full + theta - intrinsic_mean_I

    if scale_type == 'vreeswijk':
    # change susceptibility but preserve correlations and mean activities by letting sigma'/sigma=sqrt(K_scaling) and preserving (mu-theta)/sigma
        noise_std_E = np.sqrt((K_scaling - 1.)*intrinsic_var_E + K_scaling*noise_std_E_full**2)
        noise_std_I = np.sqrt((K_scaling - 1.)*intrinsic_var_I + K_scaling*noise_std_I_full**2)
        noise_mean_E = (1. - np.sqrt(K_scaling))*theta - intrinsic_mean_E + np.sqrt(K_scaling)*(noise_mean_E_full + intrinsic_mean_E_full)
        noise_mean_I = (1. - np.sqrt(K_scaling))*theta - intrinsic_mean_I + np.sqrt(K_scaling)*(noise_mean_I_full + intrinsic_mean_I_full)

    m_x = 0.
    j_EX = j_IX = K_EX = K_IX = 0.

J_array = np.array([[j_EE, j_EI, j_EX], [j_IE, j_II, j_IX]])
K_array = np.array([[K_EE, K_EI, K_EX], [K_IE, K_II, K_IX]])
mu_x = np.array([noise_mean_E, noise_mean_I])
sigma_x = np.array([noise_std_E, noise_std_I])

d = 0.1       # ms synaptic delay, small, should be equal to h
h = 0.1       # ms resolution

dt_samp = 3.*h   # sampling interval of activity

# for kickstarting an otherwise silent network
kickstart = False
ks_mean = theta
ks_std = 0.8
ks_stop = 5.

frac_rec = 0.5 # fraction of neurons to record from
N_rec_E = int(frac_rec*N)
N_rec_I = int(frac_rec*N)

record_spin = False # whether to record spins of individual neurons
n_spin = 20 # number of neurons in each population from which to record spins if record_spin==True

datapath = '../data/'

