# Parameters for binary_network_symm.py

import numpy

n_virtual_procs = 4

theta = -3.0 # mV threshold

tau_m = 10.0   # ms time constant of neuron

neuronmodel = 'mcculloch_pitts_neuron' # 'ginzburg_neuron' or 'mcculloch_pitts_neuron'

beta = 100.0   # slope for ginzburg_neuron in mV^-1. Use a high slope to obtain saturation so that tanh is effectively a Heaviside function

input_type = 'binary' # 'binary' or 'noise'

# noise input, single-step downscaling:

# full_mean = p_full*N_full*j_EE*ms[0] - g*gamma*j_EE*p_full*N_full*ms[0]
# full_var = p_full*N_full*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g**2)

# intrinsic_mean = p*N*j_EE*ms[0] - g_new*gamma*j_EE*p*N*ms[0]
# intrinsic_var = p*N*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g_new**2)

#noise_mean = -4.3263982238889005 # full_mean - intrinsic_mean
#noise_std = 1.3252676611827883 # numpy.sqrt(full_var - intrinsic_var)

# noise input, step 2 of two-step downscaling:

# step1_mean = p_full*N*j_EE*ms[0] - g_new*gamma*j_EE*p_full*N*ms[0]
# step1_var = p_full*N*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g_new**2)

# intrinsic_mean = p*N*j_EE*ms[0] - g_new*gamma*j_EE*p*N*ms[0]
# intrinsic_var = p*N*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g_new**2)

#noise_mean = -0.25816826808571436 # step1_mean - intrinsic_mean
#noise_std = 0.44785735097162915 # numpy.sqrt(step1_var - intrinsic_var)

# two-step downscaling: first only change g and the network size, and determine theta that preserves rates
# g_new = 1.2857568905538252
# new_mean = p_full*N*j_EE*ms[0] - g_new*gamma*j_EE*p_full*N*ms[0]
# -0.31978918712602256
# new_var = p_full*N*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g_new**2)
# 0.24845075892238278
# import scipy.special
# theta_new = scipy.special.erfinv(1.-2*ms[0])*numpy.sqrt(2*new_var)+new_mean 
#theta = 0.2709874099522841

# new threshold to preserve rates with van Vreeswijk scaling (J prop. to 1/sqrt(N))
# new_mean = p*N*j_EE*ms[0] - g*gamma*j_EE*p*N*ms[0]
# new_var = p*N*j_EE**2*ms[0]*(1-ms[0])*(1+gamma*g**2)
# import scipy.special
# theta_new = scipy.special.erfinv(1.-2*ms[0])*numpy.sqrt(2*new_var)+new_mean
# for 50%:
#theta = -1.5432614142578958

#full scale or step 1 of two-step downscaling:
m_x = 0. # mean activity of external neurons

T = 30000.0   # ms

# use two populations of each N neurons, exc, inh
N_full = 1000
# downscaled:
N = 1000

# connection probability
# full scale or step 1 of two-step downscaling:
p = 0.2
# downscaled: p = p_full*f*N_full/N
#p = 0.2*0.23450276379347532*N_full/N
# step 2 of two-step downscaling: p = p_full*f (but note that f needs to be recalculated between step 1 and step 2):
#p = 0.2*0.19269231581624596

K = p*N

d = 0.1       # ms synaptic delay, small, should be equal to h
h = 0.1       # ms resolution

dt_samp = 10.0*h   # sampling interval of activity

epsilon = 1./numpy.sqrt(N_full)
# van Vreeswijk scaling:
#epsilon = 1./numpy.sqrt(N)

# full scale:
j_EE = 3.*epsilon    # coupling strength E->E
j_EI = -9*epsilon  # coupling strength I->E
j_IE = 3.*epsilon    # coupling strength E->I
j_II = -9*epsilon  # coupling strength I->I
j_IX = 3.*epsilon
j_EX = 3.*epsilon

# downscaled:
#j_EE = 3.*epsilon    # coupling strength E->E
#j_EI = -3*1.2857568905538252*epsilon  # coupling strength I->E
#j_IE = 3.*epsilon    # coupling strength E->I
#j_II = -3*1.2857568905538252*epsilon  # coupling strength I->I

# for kickstarting an otherwise silent network
kickstart = False
ks_mean = theta
ks_std = 0.8
ks_stop = 5.

N_rec = int(N/2) # number of neurons to record from
N_rec_states = 100 # how many states to record
T_rec_states = 10000. # until when to record states

record_states = False # record the state variables of Nrec exc and N_rec inh neurons

indeg_dist = False   # if in-degrees of local connections should be distributed

datapath = '../data/'

suffix_stem = 'symm_fixed_low_Nx'
